
%if 0

MetaKern - a boot menu that replaces the FreeDOS kernel

Initial version from 02.11.2002
Updated 26.03.2003: Allow up to 4 boot sectors, 4 partitions, 3 drives.
Updated 01.04.2003: Autogenerate kernel names for several FreeDOS boots.
Updated 18.04.2003: Repaired and improved partition list display
Updated 04.05.2003: You can now compile with -D_CLS and -D_TIMEOUT
Updated 15.05.2003: Bugfix for the timeout option.
Updated 11.10.2003: Use "new style" with MetaKern loaded as metakern.sys
                    (you need at least SYS 2.9 / kernel 2032a or better)
Updated 16.08.2008: Added some boot sector types, added _SHORTMSG option
Updated 2018-09-22 and on: Refer to repo at https://bitbucket.org/ecm/metakern

Please read the documentation for more details! In particular:
DO NOT USE THIS IF YOU HAVE NO KNOWLEDGE OF THE DOS BOOT PROCESS.

Disclaimer:
   This is a beta version. Use it at your own risk. You may end up being
   unable to boot (then SYS again). License: GNU GPL 2. Copyright 2002-2003
   Eric Auer, eric@coli.uni-sb.de - please send feedback or patches. The
   GNU General Public License is included in the file named COPYING.
   Basically it means: Freeware and open source, but may only be included
   in projects which are open source, too.

   We're of the opinion that the GPL allows you to boot non-free operating
   systems using MetaKern (but not to distribute it with / as part of a
   non-free / non-open operating system or software package).

   Parts modified 2008 and 2018 by C. Masloch, cm -AT- bttr-software.de
   Licensed under the GNU GPL version 2, as described above.

Compile:
   nasm -O3 -o metakern.bin metakern.asm
or if you want to use any option, add -Doption for each option, eg:
   nasm -O3 -D_CLS -D_TIMEOUT -o metakern.bin metakern.asm

All options are:
   _CLSin
    Clear screen before MetaKern displays the menu
   _CLSout
    Clear screen after MetaKern's menu is done
   _CLS
    _CLSin and _CLSout
   _CLSmode
    Screen mode for _CLSin and _CLSout. 00h-FFh is used as mode for interrupt
    10h, function 00h. The low byte of 100h+ is used as value for al to call
    interrupt 10h, function 11h. Some valid values:
     01h - 40x25
     03h - 80x25
     07h - 80x25 monochrome
     110h - VGA: 80x25/EGA: 80x28
     120h - VGA: 80x43/EGA: 80x50
     140h - VGA: 80x25
   _TIMEOUT
    Set up a default option timeout:
   _TIMEOUTseconds
    Number of seconds to wait until using default
    Translated and used as _TIMEOUTticks. Some values will be rounded
    up, eg _TIMEOUTseconds=2 will wait 2 seconds and a half of one
    timer tick (this is, 1/36.4 seconds more than the specified value)
   _TIMEOUTticks
    Internal timer ticks to wait until using default (1/18.2 seconds)
    Default is 182 which is exactly 10 seconds
   _TIMEOUTtext
    Text which is displayed in the menu message describing the amount of time
    Default is translated from the value of _TIMEOUTticks
    Unused if _SHORTMSG is used
   _TIMEOUTdefault
    Default key which is actived if the timeout expires
    Default is '1'
   _WEAKFLOPPY
    Don't show partition table (of HDD) if booting FDD
   _SHOWACTIVE
    Show the active partition table entry when booting from harddisk. By default
    it's not shown because it will probably simply point to where you came from,
    and unless your MBR is broken, it will be equivalent to the "boot from C:"
    selection
   _HIDEACTIVEIFFLOPPY
    Hide the active partition if booting from FDD. By default it's shown when
    booting from FDD.
   _SHORTMSG
    Display only very short messages
    Not recommended unless you know MetaKern very well

%endif

%include "lmacros2.mac"

	defaulting

%ifndef _MAP
%elifempty _MAP
%else	; defined non-empty, str or non-str
	[map all _MAP]
%endif

	numdef CLS,			0
	numdef CLSin,			0
	numdef CLSout,			0
%if _CLS
	numdef CLSin,			1
	numdef CLSout,			1
%endif
	numdef TIMEOUT,			0
	numdef WEAKFLOPPY,		0
	numdef SHOWACTIVE,		0
	numdef HIDEACTIVEIFFLOPPY,	0
	numdef SHORTMSG,		0
	numdef NOPADDING,		0
	numdef WASTECODE,		0
	numdef OLDSTYLE,		0
	numdef IDLEHALT,		1


	cpu 8086
	org 0   ; flat binary file, gets loaded to 0x60:0 by the FreeDOS
		; boot sector, which passes on the current drive number
		; in BL and can find us, kernel.sys anywhere in the root
		; directory (may even be fragmented)...
		; stack will be at 0x1fe0:7be0 or something,
		; fat will be loaded to RAM, too. See the FreeDOS sources.

; The default values for the timeout, timeout default key and screen mode follow.
; These values are only used if either _TIMEOUT or _CLS is selected. But it doesn't
; hurt if we set them always.

%ifnnum _TIMEOUTticks
%ifnnum _TIMEOUTseconds		; only if both _TIMEOUTticks and _TIMEOUTseconds not defined!
%assign _TIMEOUTseconds 10	; 10 seconds default
%endif
%endif

%ifnum _TIMEOUTseconds		; translate seconds value to ticks. Do _not_ change this!
%assign _TIMEOUTticks ((_TIMEOUTseconds*182)+9)/10
; Adding 9 in this expression causes the value to be rounded up before dividing by 10
; Multiplying by 182 and dividing by 10 is the same as multiplying by 18.2
%endif

%ifnstr _TIMEOUTtext		; translate ticks value into message showing seconds
%assign _TIMEOUTtext _TIMEOUTticks*10/182	; translate _TIMEOUTticks (back) to seconds
%defstr _TIMEOUTtext _TIMEOUTtext		; make a string from the number
%endif

%ifnstr _TIMEOUTdefault
%define _TIMEOUTdefault '1'	; First boot sector
%endif

%ifnnum _CLSmode
%assign _CLSmode 03h		; 80x25
%endif

; Please only change %define/%assign lines ABOVE this line if you have
; no real clue on the internal workings of MetaKern. Actually, even the
; above define lines can be replaced by using the -D...=... option with
; nasm. You can select all options without editing this file.

; *************************************************************

; if you change this to have more or less than 4 boot sector menu
; entries, you must change the code, too. However, you can simply
; copy /b less than 4 boot sectors if you want to USE less entries,
; which is probably all that you need!

%define boot1 (end)
; boot2 is at (end+0x200)
; boot3 is at (end+0x400)
; boot4 is at (end+0x600)

; the MBR buffer _must_ be a multiple of 0x200 (512) after the boot1 buffer!
%define mbr (boot1+0x800)


; *************************************************************

start:
	push cs
	pop ds
	push cs
	pop es		; both ds and es point to our kernel segment: 0x0060
	mov [drive],bl		; remember drive
	mov [realdrive],bl	; remember drive
	cld
	mov cx,256
	xor ax,ax
	mov di,mbr
	repe stosw	; clear buffer for MBR

%if _WEAKFLOPPY
	or bl,bl
	jns nothard	; highest bit (0x80) clear? Then we run from floppy
			; we COULD load the MBR of drive 0x80 anyway
			; in that case (ignoring errors)
	mov dl,bl	;  Int13 expects the drive in dl, not bl
%else
	mov dl,0x80	; read MBR from harddisk C: (first harddisk) in any case
%endif

getmbr:
	mov si,3	; trial count
getmbr2:
	mov ax,0x0201	; read 1 sector
	mov cx,1	; 1st sector, 0th cylinder
	mov dh,0	; 0th head
	mov bx,mbr	; load to there
	int 0x13	; BIOS: disk
	jnc gotmbr
	xor ax,ax	; reset disk
	int 0x13
	dec si
	jnz getmbr2	; try again
	; jmp short gotmbr	; give up (no problem)
		; No jump required, can just fall through

nothard:
	; mov dl,0x80	; activate those two lines to load the MBR anyway
	; jmp short getmbr

gotmbr:
%if _CLSin
	mov ax,3	; 80x25 color
	int 0x10	; clear / reset screen mode first
%endif
	mov si,message
	call print	; print hello world


bootsectormenu:
	mov si,sectormenumsg
	call print
%if _WASTECODE
	mov al,'1'	; the key
	mov ah,1	; the bit
%else
	mov ax,(1 <<8) + '1'
		; this is the same as 1 in the high byte and '1' in the low byte,
		;  but using one instruction is shorter
%endif
	mov si,boot1	; the bootsector offset
bootmenuloop:
	call checkboot	; valid boot sector? print entry and info!
	jc noboothere
	or byte [valid],ah	; mark choice as valid
noboothere:
	inc al		; next key
	shl ah,1	; next bit
	add si,512	; next bootsector
	cmp al,'5'
	jb bootmenuloop


partitionmenu:
	cmp word [si+510],0xAA55
	jnz skipmbr	; if we could not load any mbr, skip this
	push si			; fix 18apr2003
	mov si,partnmenumsg
	call print
	pop si			; fix 18apr2003
	add si,(510-64)	; point to 1st partition table entry
	; ah already is 16 (0001_0000b)
	; al already is '5'
nextpartn:
	call checkpartn	; valid partition? print entry and info!
	jc nopartn
	or [valid],ah		; mark choice as valid
nopartn:
	inc al		; next iteration
	shl ah,1	; next iteration
	add si,16	; next partition table entry
	cmp al,'9'
	jb nextpartn	; loop
skipmbr:


abcentries:		; *** new: reboot from A:/B:/C:
	; *** we could check if the drives are actually installed here!
			; moved this text into the greeting 03may2003
	; mov si,drivemenumsg, call print
			; removed explicit list 18apr2003
	; mov byte [choice],'A', mov byte [dmaster],'A'
	; mov si,diskmaster, mov cx,3
; abcloop: call printchoice, inc byte [choice], inc byte [dmaster]
	; loop abcloop

prompt:	

; *************************************************************

	; now we have a validity mask in [valid] and a list on the
	; screen that will offer the boot sectors (1..4) and
	; the partitions (5..8), possibly not all of them.

	; only in while we wait for the FIRST keystroke from the user
	; we also check for the auto-selection / timeout things.
	; So "prompt:" runs the timeout, but "query:" does not.

%if _TIMEOUT
timer:	push ds
	mov ax,0x40
	mov ds,ax		; [40:6c] is the 32bit time of day time
	xor cx, cx		; counter of tick changes
timeloop_set_dx:
	mov dx, word [6Ch]	; get low word of current tick value
timeloop:
%if _IDLEHALT
	sti			; use "sti, hlt" (wait for IRQ)...
	hlt			; ...to save energy while waiting.
	nop
%endif
	mov ah,1
	int 0x16		; check for keystroke: ZF if none
	jnz timecancel2		; BUGFIX
	cmp dx, [6Ch]		; compare current tick value low word to prior
	je timeloop		; keep waiting if it didn't change
	inc cx			; increment counter of tick changes
	cmp cx, _TIMEOUTticks	; have observed that many tick changes ?
	jb timeloop_set_dx	; no, continue looping -->
	mov al, _TIMEOUTdefault	; assume that user wants the default
; timecancel:
	pop ds
	mov ah,[realdrive]	; the actual drive that booted us
	mov [drive],ah		; reset un-real boot drive number
	jmp short queried
timecancel2:			; BUGFIX
	pop ds			; BUGFIX (next, query: CONSUME key)
%endif				; end of the _TIMEOUT code


query:
	mov al,[realdrive]	; the actual drive that booted us
	mov [drive],al		; reset un-real boot drive number
	xor ax,ax
	int 0x16	; BIOS keyboard: get keystroke
queried:
	xor cx,cx
	mov ah,0
	cmp al,'1'	; ah is the scancode, AL the ASCII
	jb requerybeep	; ignore char
	cmp al,'8'	; 1..4 for boot sectors, 5..8 for partitions
	ja requery2	; try A..C
	sub al,'1'
	mov si,ax	; remember choice
	mov cl,al
	mov ax,1
	shl ax,cl	; bit for mask checking
	test al,[valid]
	jnz bootme
	; inactive menu entry, reject:

requerybeep:		; user pressed an invalid choice key
	mov al,7	; "beep"
	call printchar	
jmpquery_1:
	jmp short query	; ask again

requery2:		; non numeric key pressed
	or al,0x20	; make lower case
	cmp al,'a'
	jb requerybeep	; no hotkey at all
	cmp al,'c'
	ja requerybeep	; not allowed drive / no hotkey
	sub al,'a'
	jmp bootdisk	; user has selected A, B or C



bootme:
%if _SHORTMSG
	call printcrlf
%endif
	cmp si,4	; user has selected partition of boot sector
	jb bootdos	; 0..3 are sectors, 4..7 partitions
	sub si,4	; normalize to have 0..3 as selection



bootpartn:		; start a partition, assuming none are active
%ifn _SHORTMSG
	push ax
	push si
	shl si,1	; create a stamp based on the selection
	shl si,1
	add si,enumnames	; "1st 2nd 3rd 4th"
	mov ax,[si]
	mov [partnslotname],ax
	mov ax,[si+2]
	mov [partnslotname+2],ax
	mov si, partnslotmsg
	call print	; announce boot, tell which entry gets used
	pop si
	pop ax
%endif

	mov cl,4	; 16byte/entry
	shl si,cl	; translate to partition table entry
	add si,mbr+510-64	; partition table
	mov byte [si],0x80	; activate partition
	mov byte [drive],0x80	; feel like some MBR
	mov si,4	; IMPORTANT: "5th boot sector" is MBR



bootdos:		; copy and activate a stored boot sector or MBR
%ifn _SHORTMSG
	push ax
	push si
	shl si,1	; create a stamp based on the selection
	shl si,1
	add si,enumnames	; "1st 2nd 3rd 4th"
	mov ax,[si]
	mov [bootsectname],ax
	mov ax,[si+2]
	mov [bootsectname+2],ax
	mov si, bootsectmsg
	call print	; announce boot, tell which entry gets used
	pop si
	pop ax
%endif

	mov cl,9	; 512byte/sector
	shl si,cl	; translate to a choice
	add si,boot1	; here we are!

%if _WASTECODE
	xor ax,ax
	mov es,ax
	mov di,0x7C00		; We'll copy the boot sector to 0x0:0x7C00
%else
	mov di,0x7C00 - 0x600	; But we can express it shorter:
				;  relative to our "kernel" segment (0x60)
				; 0x60:(0x7C00 - 0x600) == 0x0:0x7C00
				; because seg:0x60 *0x10 = ptr:0x600 which we'll
				;  then subtract from the ptr
%endif

	mov cx,0x100
	repe movsw	; copy boot sector to usual place
	mov dl,[drive]		; see table 00653 in RBIL 61



boot7c00:
%if _CLSout
	mov ax,3	; 80x25 color
	int 0x10	; clear / reset screen mode before leaving
%endif
%ifn _SHORTMSG
	mov si,readytogomsg
	call print	; last message
%endif
			; Here we LEAVE MetaKern (for ALL menu items!):
	mov si, zeropara		; -> partition table placeholder
	mov dh,0x20			; "drive is supported by int 0x13"
	jmp word 0:0x7C00		; go for it! (and pass on DH and DL)



bootdisk:			; *** new: reboot from A:/B:/C:
%if _SHORTMSG
	call printcrlf
%else
	push ax
	add al,'A'		; drive letter: stamp into diskbootmsg
	mov [diskbootname], al		; add drive letter to template
	pop ax
%endif
	cmp al,2
	jb bootfloppy		; A: or B:
	add al,0x80-2		; C: ...
bootfloppy:
	; mov bl,al
	; mov dl,bl
	mov dl,al		; THAT drive
%ifn _SHORTMSG
	mov si,diskbootmsg
	call print		; Tell about our plans
%endif
	mov si,3		; trial count


getdisk:
	mov ax,0x0201		; read 1 sector
	mov cx,1		; 1st sector, 0th cylinder
	mov dh,0		; 0th head

%if _WASTECODE
	xor bx,bx
	mov es,bx
	mov bx,0x7C00		; We'll load the boot sector to 0x0:0x7C00
%else
	mov bx,0x7C00 - 0x600	; It's possible to express it shorter:
			;  relative to our "kernel" segment (0x60)
			; 0x60:(0x7C00 - 0x600) == 0x0:0x7C00
			; because seg:0x60 *0x10 == ptr:0x600 which we'll
			;  then subtract from the pointer
%endif

	int 0x13		; BIOS: disk
	jc getdiskerr		; retry if disk error

gotdisk:			; DL is still set

%if _WASTECODE
	cmp word [es:0x7C00 + 510],0xAA55		; Check boot code signature
%else
	cmp word [(0x7C00 + 510) - 0x600],0xAA55	; (kernel segment relative address)
			; 0x60:( (0x7C00 + 510) - 0x600) == 0:(0x7C00 + 510)
%endif

	je boot7c00		; if signature is there: jump to the boot sector, good luck!
				;  otherwise: no boot code installed, fall through to gotnodiskflag
gotnodiskflag:

%if _WASTECODE
	push cs
	pop es			; Must set it back to our code segment
%endif

requerynosystem:
	mov si,loadinvmsg
	call print	; un-bootable boot sector -  reprompt
jmpquery_2:
	jmp jmpquery_1


getdiskerr:
	xor ax,ax		; reset disk
	int 0x13
	dec si
	jnz getdisk		; try again

gotnodisk:			; give up, disk error

%if _WASTECODE
	push cs
	pop es			; Must set it back to our code segment
%endif

requeryfail:		; loading that selection did not succeed
	mov si,loadfailmsg
	call print	; disk error - reprompt
	jmp short jmpquery_2	; ask again


; *************************************************************

checkpartn:		; check and print type of partition
	push ax		; table entry at SI, deactivate entry
	push si
	mov [choice],al		; store the hotkey
	xor ax,ax
	mov al,[si]	; partition active?
	mov [si],ah	; deactivate in any case!

%ifn _HIDEACTIVEIFFLOPPY
	cmp byte [realdrive],0x80
	jb checkactivepartn	; if boot was from A: do not hide active
%endif

%ifn _SHOWACTIVE
	cmp al,0x80
	jz checkfail	; was already active, do not offer as choice!
%endif

checkactivepartn:
	mov al,[si+4]	; operating system / partition type indicator
	or al,al
	jz checkfail	; empty partition, do not offer!
	cmp al,5	; Extended
			; (d5/e1/e4 are deprecated "extended" as well)
	jz checkfail	; do not offer extended "meta" partitions
	cmp al,0x0F	; Extended LBA
	jz checkfail	;  also do not offer either this
	cmp al,0x85	; Extended Linux
	jz checkfail	;  or that
		push ax
	mov ah,al	; ah will be '0'..'F' for low nible
	and ah,0x0f
	shr al,1
	shr al,1
	shr al,1
	shr al,1
	add ax,'00'	; start conversion to ASCII
	cmp al,'9'
	jbe nibok1
	add al,7	; 'A'-('0'+10)
nibok1: cmp ah,'9'
	jbe nibok2
	add ah,7
nibok2:	mov [ptype],ax	; conversion to ASCII now done, store into message
		pop ax		; new type detection - 18apr2003
	mov ah,al		; Remember type in ah
	mov si,ptype_tbl	; table: one byte partition table type, one byte message offset
loop_tbl:
	lodsb			; Get stored type
	inc si			; point to next table entry
	cmp al,00h		; End of table marker?
	jz end_tbl
	cmp al,ah		; Does it match the type of our partition?
	jne loop_tbl

end_tbl:
	dec si			; point back into current entry
	lodsb			; get message offset from table
	mov ah,0		; high byte zero
	mov si,ax
	add si, ptypemsgs	; Add base to get full pointer
	push si
	mov si, choice
	call print		; print "N -> "
	pop si
	call print		; print partition description
	mov si,partition	; message telling the type as number
%if _SHORTMSG
	call print		; _SHORTMSG MetaKern doesn't use CR + LF here
%else
	call printstrcrlf	; print and add CR LF
%endif
	clc
	jmp short checknothing	; return, no further printing

; *************************************************************

checkfail:
	stc			; invalid choice

checkdone:			; use SI to print a menu entry
	jc checknothing		; no valid menu entry, print nothing
	pushf
	call printchoice	; SI is entry name
	popf
checknothing:
	pop si
	pop ax
	ret

; *************************************************************

checkboot:	; scan 512 bytes from SI for boot sector stuff...
	mov [choice],al		; store the hotkey
	push ax
	push si
	cmp word [si+510],0xAA55	; bootable (magic value)?
	jne checkfail		; nope! (everything else is accepted!)

	push bx
	push di
	push bp
	push cx
	push dx

	mov bp,si		; where to search
	mov bx,searchlist
	mov di,fdsearch		; first search string
checkone:
	mov ax,[di]
	or ax,ax
	jz notypefound
	call stringfind	; search in BP..BP+512, string DI, len 11
	jz typefound	; Z = hit, at offset SI, DI unchanged
nexttype:
	inc bx		; next message
	inc bx
	add di,11	; next search string
	jmp short checkone	; go on with searching
typefound:
	cmp di,fdsearch	; did we find THAT one?
	jnz notypefound	; else, no special actions needed

%ifn _SHORTMSG
	mov cx,[fdreplacecnt]		; fetch current replace snippet
	; or cx,0x2020	; turn into lower case (technically not needed
			; for new style, only for _OLDSTYLE "EL"->"el")
	mov [fdtypecnt],cx		; update snippet in entry name
%endif

	mov cx,11
	mov di,si	; overwrite (BEFORE counting up)
	mov si,fdreplace	; patch the name! (only first occurance)
	rep movsb	; replaced "kernel  sys" by a new value...

typecounting:		; count up the replacement string: first
			; value is kernel.sys (_OLDSTYLE: fdkernel.sys)
			; next are kernel2.sys, kernel3.sys etc.
			; (_OLDSTYLE: fdkern2.sys, fdkern3.sys etc.)
	push ax
	mov ax,[fdreplacecnt]

%if _OLDSTYLE
	cmp ax,"EL"	; if last was not numerical, start counting
%else
	cmp ax,"  "	; if last was not numerical, start counting
%endif

	jnz typecount1
	mov ax,"1 "	; one LESS than the first numerical suffix
typecount1:
	inc ax
	mov word [fdreplacecnt],ax	; update the name tail
	pop ax
	
notypefound:
	mov si,[bx]	; message for our type

	pop dx
	pop cx
	pop bp
	pop di
	pop bx
	clc	; SI now holds a describtion of the found boot sector
	jmp short checkdone	; print the boot menu entry and return


stringfind:	; search for string at DI, len 11, in area BP..BP+512
		; return Z for hit at offset SI, DI unchanged
	push ax
	push cx
	xor ax,ax	; start search at buffer start
restringfind:
	mov si,bp	; buffer start
	add si,ax	; current search start
	mov cx,11
	push di
	push si		; needed for "return hit offset"
	repe cmpsb	; compare strings here
	pop si		; needed for "return hit offset"
	pop di
	jz foundstring	; ZF set - found
	inc ax		; next search start
	cmp ax,512-11	; still in range?
	jb restringfind
	or ax,ax	; clear ZF - not found
foundstring:
	pop cx
	pop ax
	ret

; *************************************************************

%ifn _SHORTMSG
			; Default printchoice prints CR + LF after the message
printchoice:	; print choice + " -> " + string SI + CR + LF
	push si
	mov si,choice	; the hotkey, followed by an arrow
	call print
	pop si
%endif

printstrcrlf:		; print string SI + CR + LF
	push si
	call print	; the menu item description
	pop si
printcrlf:		; print CR + LF
	push ax
	mov al,13
	call printchar	; carriage return
	mov al,10
	call printchar	; newline
	pop ax
	retn

%if _SHORTMSG
			; The printchoice of _SHORTMSG MetaKern doesn't print CR + LF
printchoice:	; print choice + string SI
	push si
	mov si,choice	; the hotkey
	call print
	pop si
%endif

print:		; print zero-terminated string at DS:SI
	push ax	; updates SI to point to the end of the string
	; push ds
	; mov ax,cs
	; mov ds,ax		; ds is the same as cs anyway
print2:	lodsb
	or al,al
	jz eprint
	call printchar
	jmp short print2	; loop
eprint:
	; pop ds
%if _SHORTMSG
			; All messages of _SHORTMSG MetaKern want an additional space at the end
	mov al,32
	call printchar
%endif
	pop ax
	retn

printchar:	; print char from AL
	push ax
	push bx
	mov ah,0x0e	; TTY
	xor bx,bx
	int 0x10	; BIOS screen stuff
	pop bx
	pop ax
	retn

; *************************************************************

drive		db 0	; handed on to boot sectors
realdrive	db 0	; actual boot drive
valid		db 0	; bitmask of allowed choices
zeropara:	times 16 db 0

%if _SHORTMSG
	; The _SHORTMSG option mainly shortens all messages, as its name suggests
	;  The target is that the full MetaKern menu should only occupy one row,
	;  except if there's an error.


message:	db 13,10,"MetaKern - Disk: A C",0

choice:		db "X",0

sectormenumsg:	db "- System:",0

partnmenumsg:	db "- Partition:",0

partition:	db "("
ptype:		db  "xx)",0

loadfailmsg	db "Disk error, again:",0

loadinvmsg	db "No system, again:",0

%else ; _SHORTMSG

message:	db 13,10,"Welcome to the FreeDOS MetaKern boot menu"
		db 13,10,"Please type a digit to select a list item"
		db 13,10,"or type A or C to reboot from A: or C:..."
%if _TIMEOUT
		db 13,10,"Using <", _TIMEOUTdefault ,"> if no choice after "
		db _TIMEOUTtext, "s."
%endif
		db 13,10,0

choice:		db "X"		; this must be the byte before "arrow"
arrow:		db " -> ",0	; menu entries look like "X -> selection"



sectormenumsg:	db "Boot sectors:",13,10,0
partnmenumsg:	db "Partitions:",13,10,0

readytogomsg:
		db "kick!",13,10,0

				; removed explicit list 18apr2003
	; diskmaster	db "Reboot from disk "
	; dmaster	db "Z:",0

diskbootmsg	db "Booting from "
diskbootname	db "Z:"
		db "... ",0

bootsectmsg	db "Loading  "
bootsectname	db "9th "
		db " stored boot sector... ",0

partnslotmsg	db "Loading boot sector of  "
partnslotname	db "9th "
		db " partition... ",0

enumnames	db "1st 2nd 3rd 4th ",0


loadfailmsg	db "Disk read error, try again!",13,10,0

loadinvmsg	db "No system on that disk or partition, try again!",13,10,0

partition	db " - type "
ptype		db "xx",0

%endif

%include "mkostype.asm"		; 2008-08-16: provide seperate file to edit.

; *************************************************************

		; align on sector size
%ifn _NOPADDING	; (if padding enabled)
		align 512
%endif

		end:		; End of MetaKern program file

		; next sectors can be the FreeDOS and the MS-DOS boot
		; sector and so on (up to 4 boot sectors, 0x200 bytes
		; each, at 0x600, 0x800, 0xA00 and 0xC00 in the file)
		; and after that (at 0xE00) we try to load the MBR...

; *************************************************************

; use copy /b to put:
; boot1 at: end
; boot2 at: end+0x200
; boot3 at: end+0x400
; boot4 at: end+0x600
; MetaKern will load:
; mbr   at: end+0x800
